
/****************************************************************************
 Module : amsultra_d2a_simple - An AMS-Ultra connection module
 
 Input :
        d : digital equivalent of the input analog signal.

 Output :
        a : an analog output 
 
This is a simple unidirectional connection module which is used for automatic connection
module insertion during elaboration. This module polls the digital value and
creates an analog signal transition if needed. 

Algorithm:
--------
The algorithm for exact value conversion is implemented by Ultrasim
using the VPI task

Handling of Driver-Receiver Segregation:
--------
This module performs a direct mapping from digital ordinary module 
drivers to ordinary module receivers, with no delay introduced.

Compatible Disciplines:
--------
This module is intended to be used for converting signal values
from digital signals with the "logic" discipline, to analog
signals with the "electrical" discipline.

real vsup     // Supply voltage
real tr       // risetime (L/X to Hi)
real tf       // falltime (H/X to Lo)
real tx       // time to unknown (L/H/Z to X)
real tz       // impedance risetime to/from Z
real vhi      // Analog value corresponding to digital high
real vlo      // Analog value corresponding to digital low
real vx       // Analog value corresponding to digital X
real vz       // Analog value corresponding to digital Z 
real rhi      // high state output resistance
real rlo      // low state output resistance
real rx       // X state output resistance
real rz       // Z state output resistance 

Port Directions:
--------
   The output "a" is assumed to be an (output) analog port of discipline 
   "electrical", and the input "d" is assumed to be an (input) digital port 
   of discipline "logic".
 
****************************************************************************/

`include "disciplines.vams"
`timescale 1ns / 100ps

connectmodule amsultra_d2a_simple(a, d);

input d; 
output a;
electrical a; 
logic d;		

// INSTANCE PARAMETERS:
parameter real vsup=1.8 from (0:inf);       // nominal supply voltage
parameter real vlo=0;                       // output voltage levels
parameter real vhi=vsup from (vlo:vsup];    //
parameter real vthi=vhi/1.5 from (vlo:vhi); // upper threshold (def Vhi*2/3)
parameter real vtlo=vthi/2 from (vlo:vthi); // lower threshold (def Vhi*1/3)
parameter real vx=(vthi+vtlo)/2 from (vtlo:vthi); // X output voltage
parameter real vz=(vthi+vtlo)/2 from (vlo:vhi);   // Z output voltage
parameter real tr=0.2n from (0:1m);         // risetime (L/X to Hi)
parameter real tf=tr from (0:1m);           // falltime (H/X to Lo)
parameter real tx=tr from (0:1m);           // time to unknown (L/H/Z to X)
parameter real tz=tr from (0:1m);           // impedance risetime to/from Z
parameter real txdel=4*tr;                  // analog-midrange to output X
parameter real rlo=200 from (0:1M);         // low state output resistance
parameter real rhi=rlo from (0:1M);         // high state output resistance
parameter real rx=(rlo+rhi)/10 from (0:1M); // X state output resistance
parameter real rz=10M from (rhi:1T];        // Z state output resistance
parameter real vtol=vsup/100 from (0:(vthi-vtlo)/4]; // voltage tolerance
parameter real ttol=tr/4;                   // time tolerance of crossing



 	assign d = d;

	initial
		$ultra_connect_d2a(d, a, tr, tf, tx, tz, vhi, vlo, vx, vz, rhi, rlo, rx, rz); 

endmodule
